@@ -4,6 +4,6 @@ use crate::*;
 fn escape() {
     snapshot!(
         SCRIPT_ARGS,
-        &["prog", "cmda", "$foo", "`pwd`", "$(pwd)", "'", "\\1", "", "\n", "世界", " "]
+        &["prog", "cmd", "$foo", "`pwd`", "$(pwd)", "'", "\\1", "", "\n", "世界", " "]
     );
 }
